#ifndef __LFO_H__
#define __LFO_H__

/*
 * This class implements a LUT-based LFO with various waveforms and
 * linear interpolation.
 *
 * It uses 32-bit fixed-point phase and increment, where the 8 MSB
 * represent the integer part of the number and the 24 LSB the
 * fractionnal part
 *
 * Originally written Remy Muller on 2003-08-22
 *
 * Adapted for C and Specimen by that incorrigible wannabe
 * cyber-ninja, Pete Bessman, on 2004-06-03.
 */

#include <glib.h>

/* magic numbers */
enum
{
     LFO_COUNT = 4,		/* number of LFOs */
};

/* error codes */
enum
{
     LFO_ERR_ID = -1,		/* invalid id specified */
     LFO_ERR_PARAM = -2,	/* invalid parameter specified */
};

/* defines the different shapes an LFO can have */
typedef enum
{
     LFO_SHAPE_TRIANGLE,
     LFO_SHAPE_SINE,
     LFO_SHAPE_SAW,
     LFO_SHAPE_SQUARE,
}
LFOShape;

/* call me once before using any of the other commands */
void lfo_init ( );

/* I'm pretty useless without a call to this */
void lfo_set_samplerate (int rate);

/* setters */
int lfo_set_freq       (int id, float freq);
int lfo_set_shape      (int id, LFOShape shape);
int lfo_set_sync       (int id, gboolean state);
int lfo_set_sync_beats (int id, float beats);

/* getters */
int lfo_get_freq       (int id, float* val);
int lfo_get_shape      (int id, LFOShape* val);
int lfo_get_sync       (int id, gboolean* val);
int lfo_get_sync_beats (int id, float* val);

/* start all syncing LFOs and have them use the given bpm to determine
 * their frequency */
int lfo_sync (float bpm);

/* advance all LFO's phase by nframes */
void lfo_advance_all (int nframes);

/* get the current value of the lfo (no error reporting for speed) */
float lfo_get_val (int id);

#endif /* __LFO_H__ */
